@@ -29,7 +29,6 @@
 #include <sys/stat.h>
 
 #include "ephy-embed-shell.h"
-#include "ephy-embed-persist.h"
 #include "ephy-file-helpers.h"
 #include "ephy-node-common.h"
 #include "ephy-node.h"
@@ -38,6 +37,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
+#include <webkit/webkit.h>
 
 #define EPHY_FAVICON_CACHE_XML_ROOT    (const xmlChar *)"ephy_favicons_cache"
 #define EPHY_FAVICON_CACHE_XML_VERSION (const xmlChar *)"1.1"
@@ -60,7 +60,7 @@
 static void ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass);
 static void ephy_favicon_cache_init	  (EphyFaviconCache *cache);
 static void ephy_favicon_cache_finalize	  (GObject *object);
-static gboolean kill_download		  (const char*, EphyEmbedPersist*, EphyFaviconCache*);
+static gboolean kill_download		  (const char*, WebKitDownload*, EphyFaviconCache*);
 
 #define EPHY_FAVICON_CACHE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_FAVICON_CACHE, EphyFaviconCachePrivate))
 
@@ -101,7 +101,6 @@ enum
 	EPHY_NODE_FAVICON_PROP_FILENAME	 = 3,
 	EPHY_NODE_FAVICON_PROP_LAST_USED = 4,
 	EPHY_NODE_FAVICON_PROP_STATE	 = 5,
-	EPHY_NODE_FAVICON_PROP_CHECKOLD  = 6,
 	EPHY_NODE_FAVICON_PROP_CHECKED	 = 7,
 };
 
@@ -218,7 +217,7 @@ remove_obsolete_icons (EphyFaviconCache *cache,
 						 filename, NULL);
 			if (g_unlink (path) < 0)
 			{
-				g_warning ("Unable to delete %s", path);
+				LOG ("Unable to delete %s", path);
 			}
 
 			g_free (path);
@@ -362,17 +361,17 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 
 static gboolean
 kill_download (const char *key,
-	       EphyEmbedPersist *persist,
+	       WebKitDownload *download,
 	       EphyFaviconCache *cache)
 {
 	EphyNode *icon;
 
 	/* disconnect "completed" and "cancelled" callbacks */
 	g_signal_handlers_disconnect_matched
-		(persist,G_SIGNAL_MATCH_DATA , 0, 0, NULL, NULL, cache);
+		(download,G_SIGNAL_MATCH_DATA , 0, 0, NULL, NULL, cache);
 
 	/* now cancel the download */
-	ephy_embed_persist_cancel (persist);
+	webkit_download_cancel (download);
 
 	icon = g_hash_table_lookup (cache->priv->icons_hash, key);
 	g_return_val_if_fail (EPHY_IS_NODE (icon), TRUE);
@@ -399,7 +398,7 @@ delete_file (GFile *dir,
 					 NULL);
 		if (g_unlink (path) < 0)
 		{
-			g_warning ("Unable to delete %s", path);
+			LOG ("Unable to delete %s", path);
 		}
 		
 		g_free (path);
@@ -444,51 +443,87 @@ ephy_favicon_cache_finalize (GObject *object)
 	G_OBJECT_CLASS (ephy_favicon_cache_parent_class)->finalize (object);
 }
 
-static void
-favicon_download_completed_cb (EphyEmbedPersist *persist,
-			       EphyFaviconCache *cache)
+static gboolean
+is_valid_mime_type (const char *mime_type)
 {
-	const char *url;
 
-	url = ephy_embed_persist_get_source (persist);
-	g_return_if_fail (url != NULL);
+	gboolean valid = FALSE;
 
-	LOG ("Favicon cache download completed for %s", url);
+	valid = g_strcmp0 (mime_type, "image/x-ico") == 0 ||
+		g_strcmp0 (mime_type, "image/vnd.microsoft.icon") == 0 ||
+		g_strcmp0 (mime_type, "image/png") == 0 ||
+		g_strcmp0 (mime_type, "image/gif") == 0 ||
+		g_strcmp0 (mime_type, "application/octet-stream") == 0;
 
-	g_hash_table_remove (cache->priv->downloads_hash, url);
-
-	g_signal_emit (G_OBJECT (cache), signals[CHANGED], 0, url);
-
-	g_object_unref (persist);
-
-	cache->priv->dirty = TRUE;
+	return valid;
 }
 
 static void
-favicon_download_cancelled_cb (EphyEmbedPersist *persist,
-			       EphyFaviconCache *cache)
+favicon_download_status_changed_cb (WebKitDownload *download,
+				    GParamSpec *spec,
+				    EphyFaviconCache *cache)
 {
-	const char *url, *dest;
+	WebKitDownloadStatus status = webkit_download_get_status (download);
+	const char *url = webkit_download_get_uri (download);
+	const char *destination = webkit_download_get_destination_uri (download);
+	const char *mime_type;
+	GFile *file;
+	GFileInfo *file_info;
+	gboolean valid;
 
-	url = ephy_embed_persist_get_source (persist);
-	g_return_if_fail (url != NULL);
+	switch (status) {
+	case WEBKIT_DOWNLOAD_STATUS_FINISHED:
+		LOG ("Favicon cache download completed for %s", url);
 
-	LOG ("Favicon cache download cancelled %s", url);
+		g_hash_table_remove (cache->priv->downloads_hash, url);
 
-	g_hash_table_remove (cache->priv->downloads_hash, url);
+		file = g_file_new_for_uri (destination);
+		file_info = g_file_query_info (file,
+					       G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					       0, NULL, NULL);
 
-	/* remove a partially downloaded file */
-		dest = ephy_embed_persist_get_dest (persist);
-	if (g_unlink (dest) < 0)
-	{
-		g_warning ("Unable to delete %s", dest);
+		mime_type = g_file_info_get_content_type (file_info);
+		valid = is_valid_mime_type (mime_type);
+
+		if (!valid)
+		{
+			LOG ("Deleting invalid %s type", mime_type);
+			g_file_delete (file, NULL, NULL);
+		}
+		else
+		{
+			g_signal_emit (G_OBJECT (cache), signals[CHANGED], 0, url);
+		}
+
+		g_object_unref (file);
+		g_object_unref (file_info);
+
+		g_object_unref (download);
+
+		cache->priv->dirty = TRUE;
+		break;
+	case WEBKIT_DOWNLOAD_STATUS_ERROR:
+	case WEBKIT_DOWNLOAD_STATUS_CANCELLED:
+		LOG ("Favicon cache download cancelled %s", url);
+
+		g_hash_table_remove (cache->priv->downloads_hash, url);
+
+		/* FIXME: re-schedule to try again after n days? */
+		file = g_file_new_for_uri (destination);
+		if (g_file_query_exists (file, NULL))
+		{
+			LOG ("Deleting incomplete favicon download");
+			g_file_delete (file, NULL, NULL);
+		}
+
+		g_object_unref (file);
+		g_object_unref (download);
+
+		cache->priv->dirty = TRUE;
+		break;
+	default:
+		break;
 	}
-
-	/* FIXME: re-schedule to try again after n days? */
-
-	g_object_unref (persist);
-
-	cache->priv->dirty = TRUE;
 }
 
 static void
@@ -496,8 +531,10 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 			     const char *favicon_url,
 			     const char *filename)
 {
-	EphyEmbedPersist *persist;
+	WebKitNetworkRequest *request;
+	WebKitDownload *download;
 	char *dest;
+	char *dest_uri;
 
 	LOG ("Download favicon: %s", favicon_url);
 
@@ -505,42 +542,36 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 	g_return_if_fail (favicon_url != NULL);
 	g_return_if_fail (filename != NULL);
 
+	request = webkit_network_request_new (favicon_url);
+	download = webkit_download_new (request);
+	g_object_unref (request);
+
 	dest = g_build_filename (cache->priv->directory, filename, NULL);
+	dest_uri = g_filename_to_uri (dest, NULL, NULL);
 
-	persist = EPHY_EMBED_PERSIST
-		(g_object_new (EPHY_TYPE_EMBED_PERSIST, NULL));
-
-	ephy_embed_persist_set_dest (persist, dest);
-	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW |
-					       EPHY_EMBED_PERSIST_NO_CERTDIALOGS |
-					       EPHY_EMBED_PERSIST_DO_CONVERSION |
-					       EPHY_EMBED_PERSIST_NO_COOKIES
-				               );
-	ephy_embed_persist_set_max_size (persist, EPHY_FAVICON_MAX_SIZE);
-	ephy_embed_persist_set_source (persist, favicon_url);
+	webkit_download_set_destination_uri (download, dest_uri);
 
 	g_free (dest);
+	g_free (dest_uri);
 
-	g_signal_connect (G_OBJECT (persist), "completed",
-			  G_CALLBACK (favicon_download_completed_cb), cache);
-	g_signal_connect (G_OBJECT (persist), "cancelled",
-			  G_CALLBACK (favicon_download_cancelled_cb), cache);
+	g_signal_connect (G_OBJECT (download), "notify::status",
+			  G_CALLBACK (favicon_download_status_changed_cb), cache);
 
 	g_hash_table_insert (cache->priv->downloads_hash,
-			     g_strdup (favicon_url), persist);
+			     g_strdup (favicon_url), download);
 
-	ephy_embed_persist_save (persist);
+	webkit_download_start (download);
 }
 
 /**
- * ephy_favicons_cache_get:
- * @cache:
+ * ephy_favicon_cache_get:
+ * @cache: an #EphyFaviconCache
  * @url: the URL of the icon to retrieve
  * 
  * Note: This will always return %NULL for non-http URLs.
  * 
- * Return value: the site icon at @url as a #GdkPixbuf, or %NULL if
- * if could not be retrieved. Unref when you don't need it anymore.
+ * Return value: (transfer full): the site icon at @url as a #GdkPixbuf, or
+ * %NULL if if could not be retrieved. Unref when you don't need it anymore.
  */
 GdkPixbuf *
 ephy_favicon_cache_get (EphyFaviconCache *cache,
@@ -694,18 +725,16 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		{
 			return NULL;
 		}
-		valid = strcmp (mime_type, "image/x-ico") == 0 ||
-			strcmp (mime_type, "image/vnd.microsoft.icon") == 0 ||
-			strcmp (mime_type, "image/png") == 0 ||
-			strcmp (mime_type, "image/gif") == 0;
-		is_ao = strcmp (mime_type, "application/octet-stream") == 0;
+
+		valid = is_valid_mime_type (mime_type);
+		is_ao = g_strcmp0 (mime_type, "application/octet-stream") == 0;
 
 		g_object_unref (file_info);
-		g_object_unref (file);
 
-		/* As a special measure, we try to load an application/octet-stream file
-		 * as an ICO file, since we cannot detect a ICO file without .ico extension
-		 * (the mime system has no magic for it).
+		/* As a special measure, we try to load an
+		 * application/octet-stream file as an ICO file, since we
+		 * cannot detect a ICO file without .ico extension (the mime
+		 * system has no magic for it).
 		 */
 		if (is_ao)
 		{
@@ -743,16 +772,13 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		else
 		{
 			/* remove invalid file from cache */
-			/* gnome_vfs_unlink (pix_file); */
+			g_file_delete (file, NULL, NULL);
 		}
+		g_object_unref (file);
 
 		ephy_node_set_property_int (icon,
 					    EPHY_NODE_FAVICON_PROP_CHECKED,
 					    (int) checklevel);
-
-		/* epiphany 1.6 compat */
-		ephy_node_set_property_boolean
-			(icon, EPHY_NODE_FAVICON_PROP_CHECKOLD, valid);
 	}
 
 	/* if it still needs the check, mime type couldn't be checked. Deny! */
@@ -807,7 +833,7 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 
 /**
  * ephy_favicons_cache_clear:
- * @cache:
+ * @cache: the #EphyFaviconCache to clear
  * 
  * Clears the favicon cache and removes any stored icon files from disk.
  */
